// $Id: createfile.cc 3503 2009-09-21 02:37:14Z flaterco $

//  createfile:  create files for testing TA.  See also,
//  filename_tests.tar.bz2 and filedate_tests.tar.bz2

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Curse you, C99!
#define __STDC_LIMIT_MACROS

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>
#include <time.h>
#include <string.h>
#include "ta-util.hh"

void usagebarf () {
  fprintf (stderr, "Usage: createfile -s filename size-in-bytes\n"
                   "       Creates a sparse file of the specified size.\n"
                   "\n"
                   "       createfile -r filename size-in-bytes\n"
                   "       Creates a file of the specified size containing"
                   " pseudorandom filler.\n"
                   "\n"
                   "       createfile N\n"
                   "       Creates N non-sparse files of size 2000 bytes.\n");
  exit (-1);
}

void create2000 (const Dstr &fname) {
  static uint8_t z[2000];
  FILE *fp = fopen (fname.aschar(), "w");
  if (!fp) {
    perror (fname.aschar());
    exit (-1);
  }
  if (fwrite (&z, 1, 2000, fp) != 2000) {
    fprintf (stderr, "ERROR: fwrite failed on write to %s\n", fname.aschar());
    exit (-1);
  }
  if (fclose (fp) != 0) {
    perror (fname.aschar());
    exit (-1);
  }
}

// The createFile function in ta-util uses /dev/urandom, which is overkill
// for our purposes.
void cheapRandom (const Dstr &fname, ta_t size) {
  // A RAND_MAX of 2147483647 leaves us short by one bit, but that's OK.
  assert (RAND_MAX >= 2147483647);
  assert (sizeof(uint32_t) == 4);
  srand (time(NULL));
  FILE *fp = fopen64 (fname.aschar(), "w");
  if (!fp) {
    perror (fname.aschar());
    exit (-1);
  }

  ta_t bcount;
  uint32_t word;
  for (bcount = 0; bcount + 4 <= size; bcount += 4) {
    word = rand();
    if (fwrite (&word, 4, 1, fp) != 1) {
      fprintf (stderr, "Error writing to %s at position %llu\n", fname.aschar(), bcount);
      exit (-1);
    }
  }
  if (bcount < size) {
    word = rand();
    if (fwrite (&word, size - bcount, 1, fp) != 1) {
      fprintf (stderr, "Error writing to %s at position %llu\n", fname.aschar(), bcount);
      exit (-1);
    }
  }

  if (fclose (fp)) {
    perror (fname.aschar());
    exit (-1);
  }
}

int main (int argc, char **argv) {
  if (argc == 2) {
    ta_t N;
    if (sscanf (argv[1], "%llu", &N) != 1)
      usagebarf();
    for (ta_t i(0); i<N; ++i) {
      Dstr fname ("FILL");
      fname += i;
      create2000 (fname);
    }
  } else if (argc == 4) {
    ta_t size;
    Dstr fname (argv[2]);
    if (sscanf (argv[3], "%llu", &size) != 1)
      usagebarf();
    if (!strcmp (argv[1], "-s"))
      createFile (fname, size, false);
    else if (!strcmp (argv[1], "-r"))
      cheapRandom (fname, size);
    else
      usagebarf();
  } else
    usagebarf();
  return 0;
}
