// $Id: taval.cc 3863 2011-03-04 23:11:38Z flaterco $

/*
    TA (Transparent Archivist) (C) 2005 David Flater.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <assert.h>
#include <Dstr>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ta-util.hh"
#include <unistd.h>


// Returns true if file OK, false otherwise.
bool valid (hashid hash_ID, const Dstr &file, const Dstr &shash_expected) {
  Dstr actual_shash;
  try {
    getHash (hash_ID, file, actual_shash);
  } catch (FileReadException) {
    return false;
  }
  return (dstrcasecmp (actual_shash, shash_expected) == 0);
}


void usageDie() {
  fprintf (stderr, "Usage: taval [-md5 md5file] dir\n");
  exit (-1);
}


int main (int argc, char **argv) {
  printf ("\
    TA (Transparent Archivist) Version %s (C) 2005 David Flater.\n\
\n\
    This program is free software: you can redistribute it and/or modify\n\
    it under the terms of the GNU General Public License as published by\n\
    the Free Software Foundation, either version 3 of the License, or\n\
    (at your option) any later version.\n\
\n\
    This program is distributed in the hope that it will be useful,\n\
    but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
    GNU General Public License for more details.\n\
\n\
    You should have received a copy of the GNU General Public License\n\
    along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\
\n", ta_version);

  Dstr wdpath;
  bool md5 = false;
  if (argc == 2)
    wdpath = argv[1];
  else if (argc == 4) {
    if (strcmp (argv[1], "-md5"))
      usageDie();
    md5 = true;
    wdpath = argv[3];
  } else
    usageDie();
  slashterm (wdpath);

  Dstr hashfilename;
  if (md5)
    hashfilename = argv[2];
  else
    getTarget ("ta-hashes.txt", wdpath, hashfilename);
  FILE *fp = fopen64 (hashfilename.aschar(), "r");
  if (!fp) {
    perror (hashfilename.aschar());
    exit (-1);
  }

  Dstr buf;
  if (!md5)
    if (buf.getline(fp).isNull()) {
      fprintf (stderr, "Can't read hash file\n");
      exit (-1);
    }
  printf ("Reading hash file\n");
  if (!md5)
    if (buf != "1") {
      fprintf (stderr, "Hash file has unsupported version number.  Perhaps you need to upgrade TA.\n");
      fprintf (stderr, "Check http://www.flaterco.com/ta/ for the most recent version.\n");
      exit (-1);
    }

  // Read hash file into memory to reduce zigzagging of the optical drive.
  // (It still zigzags.)
  FileList flist;
  while (!(buf.getline(fp).isNull())) {
    FileStuff fs;
    buf /= fs.hash;
    if (md5) {
      assert (buf[0] == ' ' && buf[1] == ' ');
      buf /= 2;
    } else {
      assert (buf[0] == ' ');
      buf /= 1;
    }
    getTarget (buf, wdpath, fs.name);
    flist.push_back (fs);
  }
  fclose (fp);

  printf ("Checking files\n");
  bool err = false;
  const hashid hash_ID = (md5 ? MHASH_MD5 : MHASH_SHA512);
  for (FileListIndex i=0; i<flist.size(); ++i) {
    if (!(valid (hash_ID, flist[i].name, flist[i].hash))) {
      putchar ('\n');
      fprintf (stderr, "VALIDATION ERROR: %s\n", flist[i].name.aschar());
      err = true;
    }
    putchar ('.');
    fflush (stdout);
  }
  putchar ('\n');

  if (err) {
    printf ("ERRORS FOUND.\n");
    return 1;
  }
  printf ("All files OK.\n");
  return 0;
}
